// import { api, useStaff, useDevice } from "@nice/client";
import { useMainContext } from "../../layout/MainProvider";
import toast from "react-hot-toast";
import { Button, Form, Input, Modal, Select, Row, Col, Tabs, Card } from "antd";
import { useDevice } from "@nice/client";
import { useEffect, useState } from "react";
import DepartmentChildrenSelect from "@web/src/components/models/department/department-children-select";
import DepartmentSelect from "@web/src/components/models/department/department-select";
import SystemTypeSelect from "@web/src/app/main/devicepage/select/System-select";
import DeviceTypeSelect from "../select/Device-select";
import { TusUploader } from "@web/src/components/common/uploader/TusUploader";
import { api } from "@nice/client";
import type { TabsProps } from "antd";

export default function DeviceModal() {
  const {
    form,
    formValue,
    setVisible,
    visible,
    editingRecord,
    setEditingRecord,
  } = useMainContext();
  const { create, update } = useDevice();

  // 添加状态跟踪当前选择的网系类别和故障类型
  const [selectedSystemType, setSelectedSystemType] = useState<string>();
  const [selectedDeviceType, setSelectedDeviceType] = useState<string>();

  // 在组件顶层声明mutation
  const linkToDeviceMutation = api.resource.linkToDevice.useMutation();

  // 初始化时从表单获取初始值
  useEffect(() => {
    if (visible) {
      const systemType = form.getFieldValue("systemType");
      const deviceType = form.getFieldValue("deviceType");
      setSelectedSystemType(systemType);
      setSelectedDeviceType(deviceType);
    }
  }, [visible, form]);

  // 监听表单中系统类型的变化
  useEffect(() => {
    const systemType = form.getFieldValue("systemType");

    if (systemType !== selectedSystemType) {
      setSelectedSystemType(systemType);
    }
  }, [form.getFieldValue("systemType")]);

  // 监听表单中故障类型的变化
  useEffect(() => {
    const deviceType = form.getFieldValue("deviceType");

    if (deviceType !== selectedDeviceType) {
      setSelectedDeviceType(deviceType);
    }
  }, [form.getFieldValue("deviceType")]);

  const handleOk = async () => {
    try {
      const values = form.getFieldsValue();
      const { attachments = [], ...deviceData } = values;
      let deviceId;

      if (editingRecord?.id) {
        await update.mutateAsync({
          where: { id: editingRecord.id },
          data: deviceData,
        });
        deviceId = editingRecord.id;
      } else {
        const result = await create.mutateAsync(deviceData);
        deviceId = result.id;
      }

      // 如果有附件，将它们关联到此设备
      if (attachments.length > 0 && deviceId) {
        try {
          console.log("开始关联附件到设备:", deviceId, attachments);

          // 使用组件顶层声明的mutation
          for (const fileId of attachments) {
            if (!fileId) continue;

            try {
              console.log(`正在关联文件 ${fileId} 到设备 ${deviceId}`);

              // 使用已在顶层声明的mutation
              await linkToDeviceMutation.mutateAsync({
                fileId,
                deviceId,
              });

              console.log(`文件 ${fileId} 关联成功`);
            } catch (error) {
              console.error(`文件 ${fileId} 关联失败:`, error);
            }
          }
        } catch (err) {
          console.error("关联附件失败:", err);
          toast.error("附件关联失败，但设备信息已保存");
        }
      }

      toast.success(editingRecord?.id ? "更新故障成功" : "创建故障成功");
      setVisible(false);
      setEditingRecord(null);
      form.resetFields();
    } catch (error) {
      console.error("操作失败:", error);
      toast.error("操作失败");
    }
  };

  // 当模态框关闭时清空编辑状态
  const handleCancel = () => {
    setVisible(false);
    setEditingRecord(null);
    form.resetFields();
  };

  // 处理网系类别变化
  const handleSystemTypeChange = (value: string) => {
    setSelectedSystemType(value);
    // 如果网系类别变化，且故障类型不属于此网系类别，则清空故障类型
    if (selectedDeviceType) {
      form.setFieldValue("deviceType", undefined);
      setSelectedDeviceType(undefined);
    }
  };

  // 处理故障类型变化
  const handleDeviceTypeChange = (value: string) => {
    setSelectedDeviceType(value);
  };

  // 模态框标题根据是否编辑而变化
  const modalTitle = editingRecord?.id ? "编辑故障信息" : "新增故障";

  return (
    <>
      <Modal
        title={modalTitle}
        open={visible}
        onOk={handleOk}
        onCancel={handleCancel}
        width={1000}
        style={{ top: 20 }}
        bodyStyle={{ maxHeight: "calc(100vh - 200px)", overflowY: "auto" }}
      >
        <Form form={form} initialValues={formValue} layout="vertical">
          <Row gutter={16}>
            <Col span={8}>
              <Form.Item name="systemType" label="网系类别">
                <SystemTypeSelect
                  className="rounded-lg"
                  onChange={handleSystemTypeChange}
                  deviceTypeId={selectedDeviceType} // 传递当前选中的故障类型ID
                />
              </Form.Item>
            </Col>
            <Col span={8}>
              <Form.Item name="deviceType" label="故障类型">
                <DeviceTypeSelect
                  className="rounded-lg"
                  systemTypeId={selectedSystemType}
                  onChange={handleDeviceTypeChange} // 添加故障类型变化的处理函数
                />
              </Form.Item>
            </Col>
            <Col span={8}>
              <Form.Item
                name="showname"
                label="故障名称 "
                rules={[{ required: true, message: "请输入故障名称" }]}
              >
                <Input className="rounded-lg" placeholder="请输入故障名称" />
              </Form.Item>
            </Col>
          </Row>
          <Row gutter={16}>
            <Col span={8}>
              <Form.Item name="deptId" label="单位">
                <DepartmentSelect />
              </Form.Item>
            </Col>
            <Col span={8}>
              <Form.Item name="deviceStatus" label="故障状态">
                <Select className="rounded-lg" placeholder="请选择故障状态">
                  <Select.Option value="normal">已修复</Select.Option>
                  <Select.Option value="maintenance">维修中</Select.Option>
                  <Select.Option value="broken">未修复</Select.Option>
                </Select>
              </Form.Item>
            </Col>
          </Row>
          {/* 使用Card和Tabs组件合并描述和附件上传 */}
          <Card className="mt-2 rounded-lg">
            <Tabs
              defaultActiveKey="description"
              items={[
                {
                  key: "description",
                  label: "描述",
                  children: (
                    <Form.Item name="notes" noStyle>
                      <Input.TextArea
                        rows={4}
                        className="rounded-lg"
                        placeholder="请输入故障描述信息"
                      />
                    </Form.Item>
                  ),
                },
                {
                  key: "attachments",
                  label: "附件",
                  children: (
                    <Form.Item name="attachments" noStyle>
                      <TusUploader
                        multiple={true}
                        description="点击或拖拽文件到此区域上传故障相关附件"
                      />
                    </Form.Item>
                  ),
                },
              ]}
            />
          </Card>
        </Form>
      </Modal>
    </>
  );
}
