// 新建文件: apps/web/src/components/resource/resource-file-list.tsx
import { useEffect, useState, useMemo } from "react";
import {
  Empty,
  List,
  Typography,
  Button,
  Space,
  Card,
  Divider,
  Tabs,
  Image,
} from "antd";
import {
  DownloadOutlined,
  FileOutlined,
  FileTextOutlined,
  FilePdfOutlined,
  FileImageOutlined,
  EyeOutlined,
} from "@ant-design/icons";
import { api } from "@nice/client";
import { env } from "@web/src/env";
import { getCompressedImageUrl, formatFileSize } from "@nice/utils";
import { getFileIcon } from "@web/src/components/common/uploader/utils";

interface ResourceFileListProps {
  deviceId: string;
}

interface ResourceItem {
  id?: string;
  title?: string;
  fileId?: string;
  createdAt?: Date | string;
  url?: string;
  type?: string;
  meta?: any;
}

export default function ResourceFileList({ deviceId }: ResourceFileListProps) {
  // 直接在组件顶层使用useQuery
  const { data, isLoading, refetch } = api.resource.findMany.useQuery(
    {
      where: {
        deviceId: deviceId,
      },
    },
    {
      refetchOnWindowFocus: true,
      staleTime: 3000,
      enabled: !!deviceId, // 只有当deviceId存在时才执行查询
    }
  );
  useEffect(() => {
    // 每3秒刷新一次数据
    const intervalId = setInterval(() => {
      if (deviceId) {
        refetch();
      }
    }, 3000);

    return () => clearInterval(intervalId);
  }, [deviceId, refetch]);
  // 使用useMemo处理资源数据
  const { resources, imageResources, fileResources } = useMemo(() => {
    if (!data || !Array.isArray(data))
      return { resources: [], imageResources: [], fileResources: [] };

    const isImage = (url: string) => /\.(png|jpg|jpeg|gif|webp)$/i.test(url);

    // 使用显式类型断言，避免递归类型推断
    const processedResources = data.map((resource: any) => {
      const fileUrl = resource.url || resource.fileId;
      if (!fileUrl) return resource;

      const originalUrl = `http://${env.SERVER_IP}:${env.FILE_PORT}/uploads/${fileUrl}`;
      const isImageFile = isImage(fileUrl);
      // console.log("环境变量:", {
      //   SERVER_IP: env.SERVER_IP,
      //   FILE_PORT: env.FILE_PORT,
      //   fileUrl: fileUrl,
      // });
      return {
        ...resource,
        originalUrl,
        isImage: isImageFile,
      } as ResourceItem & { originalUrl: string; isImage: boolean };
    });

    return {
      resources: processedResources,
      imageResources: processedResources.filter((item) => item.isImage),
      fileResources: processedResources.filter((item) => !item.isImage),
    };
  }, [data]);

  // 获取文件URL（保留你的tabs结构，但改进URL生成逻辑）
  const getFileUrl = (fileId: string | undefined) => {
    if (!fileId) return "";

    // 使用与ResourceShower相同的URL格式
    return `http://${env.SERVER_IP}:${env.FILE_PORT}/uploads/${fileId}`;
  };

  const getCompressedUrl = (fileId: string | undefined) => {
    if (!fileId) return "";
    return getCompressedImageUrl(getFileUrl(fileId));
  };

  if (isLoading) {
    return (
      <div className="text-center py-4 bg-gray-50 rounded-md">
        加载附件中...
      </div>
    );
  }

  if (resources.length === 0) {
    return (
      <Empty description="暂无附件" className="bg-gray-50 p-4 rounded-md" />
    );
  }

  return (
    <div className="bg-gray-50 rounded-md p-3">
      <Tabs
        defaultActiveKey="all"
        items={[
          {
            key: "all",
            label: "全部附件",
            children: (
              <List
                grid={{ gutter: 16, column: 2 }}
                dataSource={resources}
                renderItem={(item: any) => (
                  <List.Item>
                    <Card
                      size="small"
                      hoverable
                      className="flex flex-col h-full"
                      actions={[
                        ...(item.isImage
                          ? [
                              <Button
                                key="preview"
                                type="link"
                                icon={<EyeOutlined />}
                                onClick={(e) => {
                                  e.stopPropagation();
                                  window.open(
                                    item.originalUrl || getFileUrl(item.fileId),
                                    "_blank"
                                  );
                                }}
                              >
                                预览
                              </Button>,
                            ]
                          : []),
                        <Button
                          key="download"
                          type="link"
                          icon={<DownloadOutlined />}
                          onClick={() => {
                            const url =
                              "originalUrl" in item
                                ? (item as any).originalUrl
                                : getFileUrl(item.fileId);
                            if (url) window.open(url, "_blank");
                          }}
                        >
                          下载
                        </Button>,
                      ]}
                    >
                      <div className="flex items-center mb-2">
                        {item.isImage ? (
                          <FileImageOutlined
                            style={{ fontSize: 24, color: "#1890ff" }}
                          />
                        ) : (
                          getFileIcon(item.url || "")
                        )}
                        <Typography.Text strong className="ml-2 truncate">
                          {item.title || `文件-${item.id}`}
                        </Typography.Text>
                      </div>
                      <Divider className="my-2" />
                      <Typography.Text type="secondary" className="text-xs">
                        文件大小:{" "}
                        {item.meta?.size
                          ? formatFileSize(item.meta.size)
                          : "未知"}
                      </Typography.Text>
                      <br />
                      <Typography.Text type="secondary" className="text-xs">
                        上传时间:{" "}
                        {item.createdAt
                          ? new Date(item.createdAt).toLocaleString()
                          : "未知"}
                      </Typography.Text>
                    </Card>
                  </List.Item>
                )}
              />
            ),
          },
          {
            key: "images",
            label: "图片附件",
            children:
              imageResources.length > 0 ? (
                <div className="p-2">
                  <Image.PreviewGroup>
                    <div className="grid grid-cols-2 md:grid-cols-3 gap-4">
                      {imageResources.map((item) => (
                        <div
                          key={item.fileId || item.id}
                          className="aspect-square rounded-lg overflow-hidden shadow-sm border border-gray-100 relative group flex items-center justify-center bg-gray-100"
                        >
                          <Image
                            src={
                              item.originalUrl || getCompressedUrl(item.fileId)
                            }
                            alt={item.title || "图片"}
                            preview={{
                              src: item.originalUrl || getFileUrl(item.fileId),
                              mask: (
                                <div className="flex items-center justify-center text-white">
                                  预览图片
                                </div>
                              ),
                            }}
                            style={{
                              width: "100%",
                              height: "100%",
                              objectFit: "contain",
                              objectPosition: "center",
                            }}
                            className="rounded-lg"
                          />
                          <div className="absolute bottom-0 left-0 right-0 bg-black bg-opacity-50 text-white p-1 text-xs opacity-0 group-hover:opacity-100 transition-opacity">
                            {item.title || `图片-${item.id}`}
                          </div>
                        </div>
                      ))}
                    </div>
                  </Image.PreviewGroup>
                </div>
              ) : (
                <Empty description="暂无图片附件" />
              ),
          },
          {
            key: "files",
            label: "文档附件",
            children:
              fileResources.length > 0 ? (
                <div className="rounded-xl p-1 border border-gray-100 bg-white">
                  <div className="flex flex-wrap gap-2">
                    {fileResources.map((item) => (
                      <a
                        key={item.fileId || item.id}
                        className="flex-shrink-0 relative active:scale-95 transition-transform select-none"
                        href={item.originalUrl || getFileUrl(item.fileId)}
                        target="_blank"
                        download={true}
                        title={`点击下载"${item.title || "未命名文件"}"`}
                      >
                        <div className="w-[120px] h-[90px] p-2 flex flex-col items-center justify-between rounded-xl hover:bg-blue-50 cursor-pointer">
                          <div className="text-blue-600 text-xl">
                            {getFileIcon(item.url || "")}
                          </div>
                          <div className="w-full text-center space-y-0.5">
                            <p className="text-xs font-medium text-gray-800 truncate">
                              {item.title?.slice(0, 12) || "未命名"}
                            </p>
                            <div className="flex items-center justify-between text-xs text-gray-500">
                              <span className="bg-gray-100 px-1 rounded-sm">
                                {item.url?.split(".").pop()?.toUpperCase() ||
                                  "文件"}
                              </span>
                              <span className="bg-gray-100 px-1 rounded-sm">
                                {item.meta?.size
                                  ? formatFileSize(item.meta.size)
                                  : ""}
                              </span>
                            </div>
                          </div>
                        </div>
                      </a>
                    ))}
                  </div>
                </div>
              ) : (
                <Empty description="暂无文档附件" />
              ),
          },
        ]}
      />
    </div>
  );
}
