import { utils, writeFile, read } from "xlsx";
import dayjs from "dayjs";
import toast from "react-hot-toast";
import { findTermIdByName, findDeptIdByName, getStatusKeyByValue } from "../utils/helpers";

export class ImportExportService {
  // 导出选中数据
  static exportSelectedData(
    selectedData: any[],
    getTermNameById: (id: string, type: string) => string
  ) {
    if (!selectedData || selectedData.length === 0) {
      toast.error("没有选中任何数据");
      return;
    }

    try {
      // 格式化导出数据
      const exportData = selectedData.map((item) => ({
        网系类别: getTermNameById(item.systemType, "system_type"),
        故障类型: getTermNameById(item.deviceType, "device_type"),
        单位: (item as any)?.department?.name || "未知",
        故障名称: item?.showname || "未命名故障",
        故障状态: (() => {
          const statusMap = {
            normal: "已修复",
            maintenance: "维修中",
            broken: "未修复",
            idle: "闲置",
          };
          return statusMap[item.deviceStatus] || "未知";
        })(),
        时间: item.createdAt
          ? dayjs(item.createdAt).format("YYYY-MM-DD")
          : "未知",
        描述: item.notes || "无描述",
      }));

      // 创建工作簿
      const wb = utils.book_new();
      const ws = utils.json_to_sheet(exportData);
      utils.book_append_sheet(wb, ws, "故障数据");

      // 导出Excel文件
      writeFile(wb, `故障数据_${dayjs().format("YYYYMMDD_HHmmss")}.xlsx`);

      toast.success(`成功导出 ${selectedData.length} 条数据`);
    } catch (error) {
      console.error("导出数据错误:", error);
      toast.error("导出失败");
    }
  }

  // 导出模板
  static exportTemplate(systemTypeTerms: any[], deviceTypeTerms: any[]) {
    try {
      // 创建一个示例记录
      const templateData = [
        {
          网系类别: systemTypeTerms?.[0]?.name || "网系类别1",
          故障类型: deviceTypeTerms?.[0]?.name || "故障类型1",
          单位: "单位名称",
          故障名称: "示例故障名称",
          故障状态: "未修复", // 可选值:已修复, 维修中, 未修复
          描述: "这是一个示例描述",
        },
      ];

      // 创建工作簿
      const wb = utils.book_new();
      const ws = utils.json_to_sheet(templateData);
      utils.book_append_sheet(wb, ws, "导入模板");

      // 导出Excel文件
      writeFile(wb, `故障导入模板.xlsx`);

      toast.success("已下载导入模板");
    } catch (error) {
      console.error("导出模板失败:", error);
      toast.error("下载模板失败");
    }
  }

  // 导入数据
  static async importData(
    file: File,
    systemTypeTerms: any[],
    deviceTypeTerms: any[],
    departments: any[],
    devices: any[],
    createRecord: (data: any) => Promise<any>
  ) {
    try {
      // 读取Excel/CSV文件
      const data = await file.arrayBuffer();
      const workbook = read(data);
      const worksheet = workbook.Sheets[workbook.SheetNames[0]];
      const jsonData = utils.sheet_to_json(worksheet);

      // 转换为符合系统格式的数据
      const records = jsonData.map((row: any) => {
        const unitName = row["单位"] ? String(row["单位"]) : "";
        return {
          systemType: findTermIdByName(row["网系类别"], systemTypeTerms),
          deviceType: findTermIdByName(row["故障类型"], deviceTypeTerms),
          showname: row["故障名称"] ? String(row["故障名称"]) : "",
          // 尝试关联部门ID
          deptId: unitName ? findDeptIdByName(unitName, departments, devices) : null,
          // 同时保存原始单位名称
          responsiblePerson: unitName,
          deviceStatus: getStatusKeyByValue(row["故障状态"]),
          notes: row["描述"] ? String(row["描述"]) : "",
        };
      });

      // 确认是否有有效数据
      if (records.length === 0) {
        toast.error("未找到有效数据");
        return false;
      }

      // 批量创建记录
      await this.batchImportRecords(records, createRecord);
      return true;
    } catch (error) {
      console.error("导入失败:", error);
      toast.error("导入失败，请检查文件格式");
      return false;
    }
  }

  // 批量导入记录
  static async batchImportRecords(  
    records: any[],
    createRecord: (data: any) => Promise<any>
  ) {
    if (!records || records.length === 0) return;
    try {
      // 过滤出有效记录
      const validRecords = records.filter(
        (record) => record.systemType || record.deviceType || record.showname
      );

      if (validRecords.length === 0) {
        toast.error("没有找到有效的记录数据");
        return;
      }
      // 设置批处理大小
      const batchSize = 5;
      let successCount = 0;
      let totalProcessed = 0;
      // 显示进度提示
      const loadingToast = toast.loading(`正在导入数据...`);
      // 分批处理数据
      for (let i = 0; i < validRecords.length; i += batchSize) {
        const batch = validRecords.slice(i, i + batchSize);

        // 串行处理每一批数据
        for (const record of batch) {
          try {
            await createRecord(record);
            successCount++;
          } catch (error) {
            console.error(
              `导入记录失败: ${record.showname || "未命名"}`,
              error
            );
          }
        }
        totalProcessed += batch.length;
        // 更新导入进度
        toast.loading(
          `已处理 ${totalProcessed}/${validRecords.length} 条数据...`,
          { id: loadingToast }
        );
      }
      toast.dismiss(loadingToast);
      // 显示结果
      if (successCount === validRecords.length) {
        toast.success(`成功导入 ${successCount} 条数据`);
      } else {
        toast.success(
          `成功导入 ${successCount}/${validRecords.length} 条数据，部分记录导入失败`
        );
      }
    } catch (error) {
      console.error("批量导入失败:", error);
      toast.error("导入过程中发生错误");
    }
  }
}

export default ImportExportService;