// apps/web/src/components/models/term/system-type-select.tsx
import { Select } from "antd";
import { api } from "@nice/client";
import React, { useEffect, useState } from "react";

interface SystemTypeSelectProps {
  value?: string;
  onChange?: (value: string) => void;
  placeholder?: string;
  disabled?: boolean;
  className?: string;
  style?: React.CSSProperties;
  deviceTypeId?: string; // 添加故障类型ID参数用于筛选
}

export default function SystemTypeSelect({
  value,
  onChange,
  placeholder = "选择网系类别",
  disabled = false,
  className,
  style,
  deviceTypeId, // 接收故障类型ID
}: SystemTypeSelectProps) {
  const [options, setOptions] = useState<{ label: string; value: string }[]>(
    []
  );
  const [loading, setLoading] = useState(false);

  // 获取所有网系类别
  const { data: allSystemTypes, isLoading: isSystemLoading } =
    api.term.findMany.useQuery({
      where: {
        taxonomy: { slug: "system_type" },
        deletedAt: null,
        parentId: null, // 只查询顶级网系类别
      },
      orderBy: { order: "asc" },
    });

  // 获取所有故障类型数据（包括父级信息）
  const { data: allDeviceTypes, isLoading: isDeviceTypesLoading } =
    api.term.findMany.useQuery({
      where: {
        taxonomy: { slug: "device_type" },
        deletedAt: null,
      },
      include: {
        parent: {
          select: {
            id: true,
            name: true,
          },
        },
      },
    });

  // 处理网系类别列表
  useEffect(() => {
    setLoading(isSystemLoading || isDeviceTypesLoading);

    if (allSystemTypes) {
      // 如果指定了故障类型
      if (deviceTypeId && allDeviceTypes) {
        // 查找故障类型对应的信息
        const deviceType = allDeviceTypes.find((dt) => dt.id === deviceTypeId);

        if (deviceType && deviceType.parentId) {
          // 找到对应的网系类别
          const parentSystemType = allSystemTypes.find(
            (st) => st.id === deviceType.parentId
          );

          if (parentSystemType) {
            // 只显示该故障类型对应的网系类别
            setOptions([
              {
                label: parentSystemType.name,
                value: parentSystemType.id,
              },
            ]);

            // 如果当前值与父级不匹配，则自动更新表单值
            if (value !== parentSystemType.id) {
              onChange?.(parentSystemType.id);
            }
          } else {
            // 如果找不到对应的网系类别，则显示所有
            setOptions(
              allSystemTypes.map((term) => ({
                label: term.name,
                value: term.id,
              }))
            );
          }
        } else {
          // 如果故障类型没有父级，显示所有网系类别
          setOptions(
            allSystemTypes.map((term) => ({
              label: term.name,
              value: term.id,
            }))
          );
        }
      } else {
        // 没有指定故障类型，显示所有网系类别
        setOptions(
          allSystemTypes.map((term) => ({
            label: term.name,
            value: term.id,
          }))
        );
      }
    }
  }, [
    allSystemTypes,
    deviceTypeId,
    allDeviceTypes,
    isSystemLoading,
    isDeviceTypesLoading,
    value,
    onChange,
  ]);

  return (
    <Select
      loading={loading}
      value={value}
      onChange={onChange}
      options={options}
      placeholder={placeholder}
      disabled={disabled || (deviceTypeId && options.length === 1)} // 如果只有一个选项且有故障类型ID，则禁用选择
      className={className}
      style={style}
      showSearch
      filterOption={(input, option) =>
        (option?.label?.toString() || "")
          .toLowerCase()
          .includes(input.toLowerCase())
      }
      allowClear
    />
  );
}
